import greenfoot.*;

public class Bug extends Actor
{
    boolean keyFoundDown;
    String mvs = "";
    
    protected void addedToWorld(World w)
    {
        int size = w.getCellSize()-4;
        getImage().scale(size, size);
    }
    
    public void act()
    {
        act2();
    }
    
    private void move(int dx, int dy)
    {
        setRotation(dx != 0 ? 90*(1-dx) : 90*dy); // turn toward intended direction of movement
        if (getOneObjectAtOffset(dx, dy, Wall.class) != null) return; // wall interference; do not move
        Mushroom mushroom = (Mushroom) getOneObjectAtOffset(dx, dy, Mushroom.class);
        if (mushroom == null || mushroom.canMove(dx, dy))
        {
            setLocation(getX()+dx, getY()+dy); // no mushroom difficulties; move
            mvs += "rdlu".charAt((1-dx)*dx*dx+(2-dy)*dy*dy);
        }
        
    }
    
    /**
     * The main issue here is the behavior of the actor moving on the upstroke of the keys; this presents
     * a feel that is not natural (having the actor move on the downstroke of the key feels better)
     */
    private void act0()
    {
        String key = Greenfoot.getKey();
        if (key == null) return;
        int dx = 0, dy = 0;
        if ("right".equals(key)) dx++;
        if ("left".equals(key)) dx--;
        if ("down".equals(key)) dy++;
        if ("up".equals(key)) dy--;
        if (dx == 0 && dy == 0) return;
        move(dx, dy);
    }        
    
    /**
     * The main issue here is when opposing keys are used, unwanted movement occurs when one is released
     */
    private void act1()
    {
        int dx = 0, dy = 0;
        if (Greenfoot.isKeyDown("right")) dx++;
        if (Greenfoot.isKeyDown("left")) dx--;
        if (Greenfoot.isKeyDown("down")) dy++;
        if (Greenfoot.isKeyDown("up")) dy--;
        if (dx*dy != 0) return; // conflicting input
        if (dx+dy == 0 && keyFoundDown) // neutral state of keys (here is where issue persists as opposing keys cancel each other)
        {
            keyFoundDown = false;
            return;
        }
        if (keyFoundDown || dx+dy == 0) return; // waiting for key release or no downstrokes detected
        keyFoundDown = true;
        move(dx, dy);
    }
    
    /**
     * There are no issues with this movement code as all key states are caught by the checking (although done in
     * a way that seems a bit strange)
     */
    private void act2()
    {
        boolean keyDown = false; // to detect current down state of any key
        int dx = 0, dy = 0;
        if (Greenfoot.isKeyDown("right")) dx++;
        if (Greenfoot.isKeyDown("down")) dy++;
        keyDown = dx+dy > 0; // positive key detection
        if (Greenfoot.isKeyDown("left")) dx--;
        if (Greenfoot.isKeyDown("up")) dy--;
        keyDown = keyDown || dx+dy < 0; // if not positive, negative key detection
        // detect change in state of keys, exit method if change is key unpressing
        if (keyDown == keyFoundDown || !(keyFoundDown = !keyFoundDown)) return;
        move(dx, dy);
    }
    /**
     * The following does the same as the act2 method above (but broke down for easier understanding of what it does)
     * 
    private void act2()
    {
        boolean keyDown = false; // for current keys state
        int dx = 0; // for horizontal direction of movement
        int dy = 0; // for vertical direction of movement
        if (Greenfoot.isKeyDown("right"))
        {
            dx++;
            keyDown = true;
        }
        if (Greenfoot.isKeyDown("left")) 
        {
            dx--;
            keyDown = true;
        }
        if (Greenfoot.isKeyDown("down"))
        {
            dy++;
            keyDown = true;
        }
        if (Greenfoot.isKeyDown("up"))
        {
            dy--;
            keyDown = true;
        }
        if (keyDown == keyFoundDown) // no change in keys state?
        {
            return;
        }
        keyFoundDown = ! keyFoundDown; // changing states
        if (!keyFoundDown) // new state is no keys pressed?
        {
            return;
        }
        
        // new state is a key is pressed
        
        // set the rotation
        if (dx == 0) // vertical movement?
        {
            setRotation(90*dy);
        }
        else // horizontal movement
        {
            setRotation(90*(1-dx));
        }
        
        // restrict movement
        if (getOneObjectAtOffset(dx, dy, Wall.class) != null) // wall obstruction?
        {
            return;
        }
        Mushroom mushroom = (Mushroom) getOneObjectAtOffset(dx, dy, Mushroom.class); // mushroom pushing?
        if (mushroom != null) // mushroom exists?
        {
            if (!mushroom.canMove(dx, dy)) // mushroom cannot move?
            {
                return;
            }
        }
        
        // move
        setLocation(getX()+dx, getY()+dy);
    }
    */
}